
/* Symmetricom Time & Frequency Processor (BC635pci) Configuration Utility
 *
 * ---
 *
 * This program provides a very simple text menu interface for accessing and
 * displaying various features of the Symmetricom Time & Frequency Processor
 * (BC635pci) board. 
 *
 * This program is based on the source code provided by Symmetricom for use on
 * Sgi Irix systems.  My understanding is that another customer of Symmetricom
 * wrote the original (bctime*.c) and it looks to be loosly based on
 * Symmetricom's code for Solaris.
 *
 * Changes in this version:
 * 
 * All code accessing the hardware has been split out into stfplib.c and
 * stfplib.h and rewritten into small functions.  The remaining code in here has
 * been rewritten to use stfplib and expaneded to cover many features of the
 * BC635pci board that weren't covered in the original code.
 *
 * David M. Cooke
 * Software Enginner, Unisys Corporation
 * Assigned to FSSB at NASA LaRC
 * d.m.cooke@larc.nasa.gov
 *
 * ---
 *
 *
 */

#include <stdio.h>
#include <time.h>
#include <unistd.h>

#include <sys/mman.h>
#include <sys/time.h>

#include "stfplib.h"


#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif


/* Menu selections:
 */
#define MENU_SET_TIMING_MODE                   1
#define MENU_SET_TIME_REGISTER_FORMAT          2
#define MENU_SET_MAJOR_TIME                    3
#define MENU_SET_YEAR                          4
#define MENU_SET_PERIODIC_OUTPUT               5
#define MENU_SET_INPUT_TIME_CODE_FORMAT        6
#define MENU_SET_INPUT_TIME_CODE_MODULATION    7
#define MENU_SET_TIMING_OFFSET                 8
#define MENU_SOFTWARE_RESET                   10
#define MENU_SELECT_CLOCK_SOURCE              11
#define MENU_CONTROL_JAM_SYNC                 12
#define MENU_FORCE_JAM_SYNC                   13
#define MENU_CONTROL_CLOCK_DISCIPLINING       14
#define MENU_LOAD_DA_CONVERTER                15
#define MENU_SET_DISCIPLINING_GAIN            16
#define MENU_SHOW_PCI_DEVICE_CONFIGURATION    17
#define MENU_SHOW_CONTROL_REGISTERS           18
#define MENU_SHOW_DPRAM                       19
#define MENU_SET_BOARD_TIME_FROM_SYSTEM       20
#define MENU_SET_SYSTEM_TIME_FROM_BOARD       21
#define MENU_SHOW_TIME                        22

#define MENU_SHOW_TIMING_MODE                 30
#define MENU_SHOW_TIME_FORMAT                 31
#define MENU_SHOW_CURRENT_YEAR                33
#define MENU_SHOW_PERIODIC_OUTPUT_SETTINGS    34
#define MENU_SHOW_TIME_CODE_FORMAT            35
#define MENU_SHOW_TIME_CODE_MODULATION        36
#define MENU_SHOW_TIMING_OFFSET               37
#define MENU_SHOW_SELECTED_CLOCK_SOURCE       38
#define MENU_SHOW_FIRMWARE_PART_NUMBER        60
#define MENU_SHOW_ASSEMBLY_PART_NUMBER        61
#define MENU_FAB_PART_NUMBER                  62
#define MENU_SHOW_MODEL_IDENTIFICATION        63
#define MENU_SHOW_SERIAL_NUMBER               64

#define MENU_TOGGLE_INTERRUPT_MASK_BITS       70
#define MENU_ZERO_INTERRUPT_COUNT             71
#define MENU_SHOW_INTERRUPT_COUNT             72

#define MENU_EXIT                              0



/* The following information may be found by looking in
 * the "/hw/.id/pci" directory on the target machine after
 * the bc635pci card has been installed.  Look for the
 * BC_VENDOR_ID and BC635_DEVICE_ID values.  Within this
 * directory will be a link to a path that might be the
 * same or similar as the one(s) below 
 */

const char*  stfp_base_pathname = "/hw/module/001c02/Ibrick/xtalk/14/pci/1/ppb_16/0";



static long  interrupt_count = 0;

static void intrfunc(void* arg)
{
  struct stfp_device*  stfp = (struct stfp_device*)arg;

  interrupt_count++;
  stfp_clear_interrupt_status(stfp, 0xffffffff);
}



int DatumTFPCommadMenu(struct stfp_device*  stfp)
{
  int  selected_option;

  printf("\n");
  printf("---------------------------------------------\n");
  printf(" BC635pci - DPRAM Configuration Command Menu \n");
  printf("---------------------------------------------\n");
  printf("Select TFP Timing Mode ..................(%2d)\n", MENU_SET_TIMING_MODE);
  printf("Select Timing Format ....................(%2d)\n", MENU_SET_TIME_REGISTER_FORMAT);
  printf("Set Major Time  .........................(%2d)\n", MENU_SET_MAJOR_TIME);
  printf("Set Year ................................(%2d)\n", MENU_SET_YEAR);
  printf("Set Periodic Output .....................(%2d)\n", MENU_SET_PERIODIC_OUTPUT);
  printf("Select Time Code Format .................(%2d)\n", MENU_SET_INPUT_TIME_CODE_FORMAT);
  printf("Select Time Code Modulation type.........(%2d)\n", MENU_SET_INPUT_TIME_CODE_MODULATION);
  printf("Set Timing Offset  ......................(%2d)\n", MENU_SET_TIMING_OFFSET);
  printf("Software Reset  .........................(%2d)\n", MENU_SOFTWARE_RESET);
  printf("Select Clock Source......................(%2d)\n", MENU_SELECT_CLOCK_SOURCE);
  printf("Control Jam Sync  .......................(%2d)\n", MENU_CONTROL_JAM_SYNC);
  printf("Force Jam Sync  .........................(%2d)\n", MENU_FORCE_JAM_SYNC);
  printf("Control Clock Disciplining  .............(%2d)\n", MENU_CONTROL_CLOCK_DISCIPLINING);
  printf("Load D/A Convertor ......................(%2d)\n", MENU_LOAD_DA_CONVERTER);
  printf("Set Disciplining Gain  ..................(%2d)\n", MENU_SET_DISCIPLINING_GAIN);
  printf("Dump PCI Configuration Register .........(%2d)\n", MENU_SHOW_PCI_DEVICE_CONFIGURATION);
  printf("Dump Control Resgisters .................(%2d)\n", MENU_SHOW_CONTROL_REGISTERS);
  printf("Dump Dual Port RAM ......................(%2d)\n", MENU_SHOW_DPRAM);
  printf("Set Board Time from System ..............(%2d)\n", MENU_SET_BOARD_TIME_FROM_SYSTEM);
  printf("Set System Time from Board ..............(%2d)\n", MENU_SET_SYSTEM_TIME_FROM_BOARD);
  printf("Show Time ...............................(%2d)\n", MENU_SHOW_TIME);

  printf("Show Timing Mode ........................(%2d)\n", MENU_SHOW_TIMING_MODE);
  printf("Show Time Format ........................(%2d)\n", MENU_SHOW_TIME_FORMAT);
  printf("Show Current Year........................(%2d)\n", MENU_SHOW_CURRENT_YEAR);
  printf("Show Periodic Output Settings ...........(%2d)\n", MENU_SHOW_PERIODIC_OUTPUT_SETTINGS);
  printf("Show Time Code Format ...................(%2d)\n", MENU_SHOW_TIME_CODE_FORMAT);
  printf("Show Time Code Modulation ...............(%2d)\n", MENU_SHOW_TIME_CODE_MODULATION);
  printf("Show Timing Offset ......................(%2d)\n", MENU_SHOW_TIMING_OFFSET);
  printf("Show Selected Clock Source ..............(%2d)\n", MENU_SHOW_SELECTED_CLOCK_SOURCE);
  printf("Show Firmware Part Number ...............(%2d)\n", MENU_SHOW_FIRMWARE_PART_NUMBER);
  printf("Show Assembly Part Number ...............(%2d)\n", MENU_SHOW_ASSEMBLY_PART_NUMBER);
  printf("Show Fab Part Number ....................(%2d)\n", MENU_FAB_PART_NUMBER);
  printf("Show Model Identification ...............(%2d)\n", MENU_SHOW_MODEL_IDENTIFICATION);
  printf("Show Serial Number ......................(%2d)\n", MENU_SHOW_SERIAL_NUMBER);

  printf("Toggle Interrupt Mask Bits ..............(%2d)\n", MENU_TOGGLE_INTERRUPT_MASK_BITS);
  printf("Zero Interrupt Count ....................(%2d)\n", MENU_ZERO_INTERRUPT_COUNT);
  printf("Show Interrupt Count ....................(%2d)\n", MENU_SHOW_INTERRUPT_COUNT);

  printf("Exit ....................................( 0)\n");
  printf("Select Your Option ......................(");

  scanf("%d", &selected_option);
  printf("\n");

  switch (selected_option)
  {
    case MENU_SET_TIMING_MODE :
    {
      int  selected_mode = 0;

      printf("  Timing Mode\n");
      printf("  ----------------------------------\n");
      printf("  Time Code - IRIG B ............(1)\n");
      printf("  Free Running - 10MHz (I/E) ....(2)\n");
      printf("  1 PPS - External One PPS.......(3)\n");
      printf("  GPS - GPS Antennal/Receiver ...(4)\n");
      printf("  Exit ..........................(0)\n");
      printf("  Select Your Option ............(");

      scanf("%d", &selected_mode);

      switch (selected_mode) 
      {
        case 1 : 
          stfp_set_timing_mode(stfp, MODE_TIMECODE);
          break;
        case 2 :
          stfp_set_timing_mode(stfp, MODE_FREERUN);
          break;
        case 3 :
          stfp_set_timing_mode(stfp, MODE_EXT1PPS);
          break;
        case 4 : 
          stfp_set_timing_mode(stfp, MODE_GPS);
          break;
        default :
          break;
      }
    }
    break;

    case MENU_SET_TIME_REGISTER_FORMAT :
    {
      int  selected_format = 0;

      printf("  Time Format\n");
      printf("  --------------------\n");
      printf("  Decimal ........ (1)\n");
      printf("  Binary  ........ (2)\n");
      printf("  Exit ............(0)\n");
      printf("  Select Option ...(");

      scanf("%d", &selected_format);

      switch (selected_format)
      {
        case 1 :
          stfp_set_time_register_format(stfp, TIME_DECIMAL);
          break;
        case 2 :
          stfp_set_time_register_format(stfp, TIME_BINARY);
          break;
        default :
          break;
      }
    }
    break;

    case MENU_SET_MAJOR_TIME :
    { 
      unsigned long  bintim; 

      printf("  Set Major Time  \n");
      printf("  ----------------\n");
      printf("  Enter Binary time to load in HEX formm (8-Digits Max):");
      scanf("%08x", &bintim);

      stfp_set_major_time_unix(stfp, (time_t)bintim);
    }
    break;   

    case MENU_SET_YEAR      :
    {
      int  year = 1999;

      printf("  Set Year\n");
      printf("  --------------\n");
      printf("  Enter Year to load (1970-2050) :");

      scanf("%d", &year);

      stfp_set_year(stfp, year);
    }
    break; 

    case MENU_SET_PERIODIC_OUTPUT :
    {
      int           sync_option;
      int           quit = FALSE;
      uint8_t       sync_flag;
      unsigned int  divider_n1;
      unsigned int  divider_n2;

      printf("  Set Periodic Output\n");
      printf("  -------------------------------\n");
      printf("  Do not Sync to 1 Sec PPS ...(1)\n");
      printf("  Sync to 1 Sec PPS ..........(2)\n");
      printf("  Exit .......................(0)\n");
      printf("  Select Your Option .........(");

      scanf("%d", &sync_option);
      switch (sync_option)
      {
        case 1 :
          sync_flag = 0;
          break;
        case 2 :
          sync_flag = 1;
          break;
        default :
          quit = TRUE;
          break;
      }

      if (quit)
        break;

      printf("  Enter value for Divider n1 (2-65535):");
      scanf("%d", &divider_n1);
      if (divider_n1 < 2 || divider_n1 > 65535)
        break;

      printf("  Enter value for Divider n2 (2-65535):");
      scanf("%d", &divider_n2);
      if (divider_n2 < 2 || divider_n2 > 65535)
        break;

      stfp_set_periodic_output(stfp, sync_flag, divider_n1, divider_n2);
    }
    break; 

    case MENU_SET_INPUT_TIME_CODE_FORMAT :
    {
      int  format_option;

      printf("  Set Time Code Format\n");
      printf("  -------------------------\n");
      printf("  IRIG-A  ..............(1)\n");
      printf("  IRIG-B  ..............(2)\n");
      printf("  2137    ..............(3)\n");
      printf("  NASA36  ..............(4)\n");
      printf("  XR3  .................(5)\n");
      printf("  Exit .................(0)\n");
      printf("  Select Your Option ...(");
      scanf("%d", &format_option);

      switch (format_option)
      {
        case 1 :
          stfp_set_input_time_code_format(stfp, TC_IRIGA);
          break;
        case 2 :
          stfp_set_input_time_code_format(stfp, TC_IRIGB);
          break;
        case 3 :
          stfp_set_input_time_code_format(stfp, TC_2137);
          break;
        case 4 :
          stfp_set_input_time_code_format(stfp, TC_NASA36);
          break;
        case 5 :
          stfp_set_input_time_code_format(stfp, TC_XR3);
          break;
        default :
          break;
      }
    }
    break; 

    case MENU_SET_INPUT_TIME_CODE_MODULATION :
    {
      int  modulation_type;
      printf("  Set Time Code Modulation\n");
      printf("  ---------------------------");
      printf("  Amplitude Modulated.....(1)\n");
      printf("  Pulse Code Moodulated...(2)\n");
      printf("  Exit ...................(0)\n");
      printf("  Select Your Option .....(\n");

      scanf("%d", &modulation_type);

      switch (modulation_type)
      {
        case 1 :
          stfp_set_input_time_code_modulation_type(stfp, MOD_AM);
          break;
        case 2 :
          stfp_set_input_time_code_modulation_type(stfp, MOD_DCLS);
          break;
        default :
          break;
      }
    }
    break; 

    case MENU_SET_TIMING_OFFSET :
    {
      long  offset;

      printf("  Set Timing Offset\n");
      printf("  -------------------\n");
      printf("  Enter Time Offset in Microseconds(-9,999,9999 to +9,999,999): ");

      scanf("%ld", &offset);

      stfp_set_propagation_delay_compensation(stfp, offset);
    }
    break; 

    case MENU_SHOW_TIMING_MODE :
    {
      uint8_t  timing_mode;

      stfp_request_data_timing_mode(stfp, &timing_mode);
      printf("Timing Mode: %s\n", stfp_timing_mode_string(timing_mode));
    }
    break;

    case MENU_SHOW_TIME_FORMAT :
    {
      uint8_t  time_format;

      stfp_request_data_time_format(stfp, &time_format);
      printf("Time Format: %s\n", stfp_time_format_string(time_format));
    }
    break;

    case MENU_SHOW_CURRENT_YEAR :
    {
      uint16_t  year;

      stfp_request_data_current_year(stfp, &year);
      printf("Year: %04d\n", year);
    }
    break;

    case MENU_SHOW_PERIODIC_OUTPUT_SETTINGS :
    {
      uint8_t   synch;
      uint16_t  divider_n1;
      uint16_t  divider_n2;

      stfp_request_data_periodic_output(stfp, &synch, &divider_n1, &divider_n2);
      printf("Periodic Output: synch=%s, n1=%u, n2=%u\n",
             synch ? "true" : "false",
             divider_n1,
             divider_n2);
    }
    break;

    case MENU_SHOW_TIME_CODE_FORMAT :
    {
      uint8_t  format;

      stfp_request_data_time_code_format(stfp, &format);
      printf("Time Code Format: %s\n", stfp_time_code_format_string(format));
    }
    break;

    case MENU_SHOW_TIME_CODE_MODULATION :
    {
      uint8_t  modulation;

      stfp_request_data_time_code_modulation(stfp, &modulation);
      printf("Time Code modulation: %s\n", stfp_time_code_modulation_string(modulation));
    }
    break;

    case MENU_SHOW_TIMING_OFFSET :
    {
      int32_t  offset;

      stfp_request_data_timing_offset(stfp, &offset);
      printf("Timing Offset: %d\n", offset);
    }
    break;

    case MENU_SHOW_SELECTED_CLOCK_SOURCE :
    {
      uint8_t  clock_source;

      stfp_request_data_selected_clock_source(stfp, &clock_source);
      printf("Selected Clock Source: %s\n", stfp_selected_clock_source_string(clock_source));
    }
    break;

    case MENU_SHOW_FIRMWARE_PART_NUMBER :
    {
      char  firmware_part_number[16];

      stfp_request_data_firmware_part_number(stfp, firmware_part_number);
      printf("Firmware Part Number: %s\n", firmware_part_number);
    }
    break;

    case MENU_SHOW_ASSEMBLY_PART_NUMBER :
    {
      uint16_t  assembly_part_number;

      stfp_request_data_assembly_part_number(stfp, &assembly_part_number);
      printf("Assembly Part Number: %u\n", assembly_part_number);
    }
    break;

    case MENU_FAB_PART_NUMBER :
    {
      uint16_t  fab_part_number;

      stfp_request_data_fab_part_number(stfp, &fab_part_number);
      printf("Fab Part Number: %u\n", fab_part_number);
    }
    break;

    case MENU_SHOW_MODEL_IDENTIFICATION :
    {
      char  model_identification[16];

      if (!stfp_request_data_model_identification(stfp, model_identification))
      {
        printf("stfp_request_model_identification() failed.\n");
        break;
      }

      printf("Model Identification: %s\n", model_identification);
    }
    break;

    case MENU_SHOW_SERIAL_NUMBER :
    {
      uint32_t  serial_number;

      stfp_request_data_serial_number(stfp, &serial_number);
      printf("Serial Number: 0x%x\n", serial_number);
    }
    break;

    case MENU_SOFTWARE_RESET :
    {
      stfp_software_reset(stfp);
    }
    break; 

    case MENU_SELECT_CLOCK_SOURCE :
    {
      int  clock_source_selection;

      printf("  Select Clock Source\n");
      printf("  ---------------------------\n");
      printf("  Internal 10  Mhz VCXO...(1)\n");
      printf("  External 10 Mhz Clock...(2)\n");
      printf("  Exit ...................(0)\n");
      printf("  Select Your Option .....(");

      scanf("%d", &clock_source_selection);

      switch (clock_source_selection)
      {
        case 1 :
          stfp_select_clock_source(stfp, SELCLOCK_INT);
          break;
        case 2 :
          stfp_select_clock_source(stfp, SELCLOCK_EXT);
          break;
        default :
          break;
      }
    }
    break; 

    case MENU_CONTROL_JAM_SYNC :
    {
      int  jam_sync_selection;

      printf("  Control Jam Sync\n");
      printf("  -------------------------\n");
      printf("  Jam Sync  Disable ....(1)\n");
      printf("  Jam Sync  Enable .....(2)\n");
      printf("  Exit .................(0)\n");
      printf("  Select Your Option ...(");

      scanf("%d", &jam_sync_selection);

      switch (jam_sync_selection)
      {
        case 1 :
          stfp_control_jam_sync(stfp, JAMSYNC_DIS);
          break;
        case 2 :
          stfp_control_jam_sync(stfp, JAMSYNC_EN);
          break;
        default :
          break;
      }
    }
    break; 

    case MENU_FORCE_JAM_SYNC :
    {
      stfp_force_jam_sync(stfp);
    }
    break; 

    case MENU_CONTROL_CLOCK_DISCIPLINING :
    {
      int  discipline_selection;

      printf("  Control Clock Disciplining\n");
      printf("  ----------------------------");
      printf("  Disciplining  Disable ...(1)\n");
      printf("  Disciplining  Enable ....(2)\n");
      printf("  Exit ....................(0)\n");
      printf("  Select Your Option ......(");
      scanf("%d", &discipline_selection);

      switch (discipline_selection)
      {
        case 1 :
          stfp_control_clock_disciplining(stfp, DISCIPLINE_DISABLE);
          break;
        case 2 :
          stfp_control_clock_disciplining(stfp, DISCIPLINE_DISABLE);
          break;
        default :
          break;
      }
    }
    break; 

    case MENU_LOAD_DA_CONVERTER :
    {
      uint16_t  dac_value;

      printf("  Load  D-A Convertor");
      printf("  -------------------");
      printf("  Enter value in Hex between 0000 (0.5V) and FFFF (4.5V): ");

      scanf("%4hX", &dac_value); 

      stfp_load_da_converter(stfp, dac_value);
    }
    break; 

    case MENU_SET_DISCIPLINING_GAIN :
    {
      short  disciplining_gain;

      printf("  Set Disciplining Gain\n");
      printf("  ---------------------\n");
      printf("  Enter  Value -32768 to +32767): ");

      scanf("%05hd", &disciplining_gain); 

      stfp_set_disciplining_gain(stfp, disciplining_gain);
    }
    break; 

    case MENU_SHOW_PCI_DEVICE_CONFIGURATION :
      print_PCI_device_configuration(stfp);
      break;

    case MENU_SHOW_CONTROL_REGISTERS :
      print_control_registers(stfp);
      break; 

    case MENU_SHOW_DPRAM :
      print_DPRAM(stfp);
      break; 

    case MENU_SET_BOARD_TIME_FROM_SYSTEM :
    { 
      struct timeval   system_time;
      struct timezone  tz;

      if (gettimeofday(&system_time, &tz) != 0)
      {
        perror("Not able to get time of day\n");
        break;
      }

      stfp_set_major_time_unix(stfp, system_time.tv_sec);

      printf(" System Time : %s\n", ctime(&system_time.tv_sec));
      printf("   tv_sec : %ld, tv_usec: %ld\n",
             system_time.tv_sec,
             system_time.tv_usec);
    }
    break;   

    case MENU_SET_SYSTEM_TIME_FROM_BOARD :
    {
      struct timeval   system_time;
      struct timezone  tz;
      uint32_t         board_time[2];

      if (gettimeofday(&system_time, &tz) != 0)
      {
        perror("Not able to get time of day\n");
        break;
      }

      printf(" System Time (before): %s\n", ctime(&system_time.tv_sec));
      printf("   tv_sec : %ld, tv_usec: %ld\n",
             system_time.tv_sec,
             system_time.tv_usec);

      stfp_time_request(stfp);
      stfp_get_time_registers(stfp, board_time);
      system_time.tv_sec  = stfp_btime_get_seconds(board_time);
      system_time.tv_usec = stfp_btime_get_usecs(board_time);
    
      if (settimeofday(&system_time, 0) != 0)
      {
        perror("Not able to set time of day\n");
      }

      if (gettimeofday(&system_time, &tz) != 0)
      {
        perror("Not able to get time of day\n");
        break;
      }

      printf(" System Time (after): %s\n", ctime(&system_time.tv_sec));
      printf("   tv_sec : %ld, tv_usec: %ld\n",
             system_time.tv_sec,
             system_time.tv_usec);
    }
    break;

    case MENU_SHOW_TIME     :
    {
      time_t    system_time;
      uint32_t  board_time[2];
      char      system_time_string[30];
      char      board_time_string[30];

      system_time = time(NULL);
      strftime(system_time_string, sizeof(system_time_string) - 1,
               "%c", localtime(&system_time));

      stfp_time_request(stfp);
      stfp_get_time_registers(stfp, board_time);
      strftime(board_time_string, sizeof(board_time_string) - 1,
               "%c", localtime((time_t*)&board_time[1]));

      printf(" System Time : %s (0x%08x) \n", system_time_string, system_time);
      printf(" Board Time  : %s (0x%08x) \n", board_time_string, board_time[1]);
    }
    break; 

    case MENU_TOGGLE_INTERRUPT_MASK_BITS :
    {
      uint32_t  interrupt_mask;
      int       mask_choice;

      while (1)
      {
        interrupt_mask = stfp_get_interrupt_enable_mask(stfp);

        printf("  Interrupt Enable Mask\n");
        printf("  ----------------------------------------------------------\n");
        printf("  EVENT_INPUT ..........................................(1)%s\n",
               (interrupt_mask & STFP_INTERRUPT_EVENT_INPUT) ? "*" : " ");
        printf("  PERODIC_OUTPUT .......................................(2)%s\n",
               (interrupt_mask & STFP_INTERRUPT_PERODIC_OUTPUT) ? "*" : " ");
        printf("  TIME_COINCIDENCE_STROBE ..............................(3)%s\n",
               (interrupt_mask & STFP_INTERRUPT_TIME_COINCIDENCE_STROBE) ? "*" : " ");
        printf("  1_PPS_OUTPUT .........................................(4)%s\n",
               (interrupt_mask & STFP_INTERRUPT_1_PPS_OUTPUT) ? "*" : " ");
        printf("  GPS_DATA_PACKET_AVAILABLE ............................(5)%s\n",
               (interrupt_mask & STFP_INTERRUPT_GPS_DATA_PACKET_AVAILABLE) ? "*" : " ");
        printf("  EVENT_IN_LOGIC_LEVEL .................................(6)%s\n",
               (interrupt_mask & STFP_INTERRUPT_EVENT_IN_LOGIC_LEVEL) ? "*" : " ");
        printf("  DCLS_IN_LOGIC_LEVEL ..................................(7)%s\n",
               (interrupt_mask & STFP_INTERRUPT_DCLS_IN_LOGIC_LEVEL) ? "*" : " ");
        printf("  EXTERNAL_1_PPS_IN_LOGIC_LEVEL ........................(8)%s\n",
               (interrupt_mask & STFP_INTERRUPT_EXTERNAL_1_PPS_IN_LOGIC_LEVEL) ? "*" : " ");
        printf("  Done .................................................(0)\n");
        printf("  Select an interrupt enable bit to toggle..............(");
        scanf("%d", &mask_choice);

        if (mask_choice == 0)
          break;

        switch (mask_choice)
        {
          case 1:
            stfp_set_interrupt_enable_mask(stfp,
                                           interrupt_mask ^ STFP_INTERRUPT_EVENT_INPUT);
            break;

          case 2:
            stfp_set_interrupt_enable_mask(stfp,
                                           interrupt_mask ^ STFP_INTERRUPT_PERODIC_OUTPUT);
            break;

          case 3:
            stfp_set_interrupt_enable_mask(stfp,
                                           interrupt_mask ^ STFP_INTERRUPT_TIME_COINCIDENCE_STROBE);
            break;

          case 4:
            stfp_set_interrupt_enable_mask(stfp,
                                           interrupt_mask ^ STFP_INTERRUPT_1_PPS_OUTPUT);
            break;

          case 5:
            stfp_set_interrupt_enable_mask(stfp,
                                           interrupt_mask ^ STFP_INTERRUPT_GPS_DATA_PACKET_AVAILABLE);
            break;

          case 6:
            stfp_set_interrupt_enable_mask(stfp,
                                           interrupt_mask ^ STFP_INTERRUPT_EVENT_IN_LOGIC_LEVEL);
            break;

          case 7:
            stfp_set_interrupt_enable_mask(stfp,
                                           interrupt_mask ^ STFP_INTERRUPT_DCLS_IN_LOGIC_LEVEL);
            break;

          case 8:
            stfp_set_interrupt_enable_mask(stfp,
                                           interrupt_mask ^ STFP_INTERRUPT_EXTERNAL_1_PPS_IN_LOGIC_LEVEL);
            break;
        }
      }
    }
    break;

    case MENU_ZERO_INTERRUPT_COUNT :
      interrupt_count = 0;
      break;

    case MENU_SHOW_INTERRUPT_COUNT :
    {
      printf(" Interrupt Count = %d", interrupt_count);
    }
    break;

    case MENU_EXIT :
      return 0;

    default :
      break;
  }

#ifdef DEBUG 
  printf("\n%d", selected_option);
#endif
  getchar();
  return 1;
}



main(int argc, char* argv[])
{ 
  struct stfp_device*  stfp;

  printf("Symmetricom Time & Frequency Processor (BC635pci) Utility\n");
  printf("---------------------------------------------------------\n" );

  /* Paging while in the ULI interrupt handler would be fatal so lock entire
   * address space in memory.
   */
  if (mlockall(MCL_CURRENT | MCL_FUTURE))
  {
   perror("mlockall");
  }

  stfp = new_stfp_device(stfp_base_pathname);
  if (stfp == 0)
  {
    printf("Failed to access bc635pci board at %s\n", stfp_base_pathname);
    exit(1);
  }

  /* Make sure interrupts are off.
   */
  stfp_set_interrupt_enable_mask(stfp, 0);
  stfp_clear_interrupt_status(stfp, 0xffffffff);

  if (!stfp_register_interrupt_function(stfp,
                                        intrfunc,
                                        (void*)stfp))
  {
    perror("Error registering interrupt function");
    exit(1);
  }

  while (DatumTFPCommadMenu(stfp)); 

  delete_stfp_device(&stfp);
}
