

struct stfp_device;


#define STFP_INTERRUPT_EVENT_INPUT                    0x01000000
#define STFP_INTERRUPT_PERODIC_OUTPUT                 0x02000000
#define STFP_INTERRUPT_TIME_COINCIDENCE_STROBE        0x04000000
#define STFP_INTERRUPT_1_PPS_OUTPUT                   0x08000000
#define STFP_INTERRUPT_GPS_DATA_PACKET_AVAILABLE      0x10000000
#define STFP_INTERRUPT_EVENT_IN_LOGIC_LEVEL           0x00200000
#define STFP_INTERRUPT_DCLS_IN_LOGIC_LEVEL            0x00400000
#define STFP_INTERRUPT_EXTERNAL_1_PPS_IN_LOGIC_LEVEL  0x00800000



uint32_t  stfp_get_interrupt_enable_mask(struct stfp_device*  stfp);

void  stfp_set_interrupt_enable_mask(struct stfp_device*  stfp, uint32_t  mask);

uint32_t  stfp_get_interrupt_status(struct stfp_device*  stfp);

void  stfp_clear_interrupt_status(struct stfp_device*  stfp, uint32_t  mask);

void stfp_time_request(struct stfp_device*  stfp);

void stfp_get_time_registers(struct stfp_device*  stfp, uint32_t*  time_registers);

time_t stfp_btime_get_seconds(uint32_t*  time_registers);

unsigned int stfp_btime_get_usecs(uint32_t*  time_registers);

unsigned int stfp_btime_get_100ns(uint32_t*  time_registers);

unsigned int stfp_time_status(uint32_t*  time_registers);

const char*  stfp_timing_mode_string(uint8_t  timing_mode);

const char*  stfp_time_format_string(uint8_t  time_format);

const char*  stfp_time_code_format_string(uint8_t  format);

const char*  stfp_time_code_modulation_string(uint8_t  modulation);

const char*  stfp_selected_clock_source_string(uint8_t  clock_source);

struct stfp_device*  new_stfp_device(const char*  base_device_name);

void delete_stfp_device(struct stfp_device**  device_pointer);

void print_PCI_device_configuration(struct stfp_device*  stfp);

void print_control_registers(struct stfp_device*  stfp);

void print_DPRAM(struct stfp_device*  stfp);

void stfp_send_command(struct stfp_device*  stfp,
                       uint8_t              command,
                       const uint8_t*       argument_buffer,
                       int                  nbytes);

int stfp_set_timing_mode(struct stfp_device*  stfp,
                         uint8_t              timing_mode);

int stfp_set_time_register_format(struct stfp_device*  stfp,
                                  uint8_t              time_register_format);

int  stfp_set_major_time_bcd(struct stfp_device*  stfp,
                             uint16_t             year,
                             uint16_t             days,
                             uint8_t              hours,
                             uint8_t              minutes,
                             uint8_t              seconds);

int  stfp_set_year(struct stfp_device*  stfp,
                   uint16_t             year);

int  stfp_set_periodic_output(struct stfp_device*  stfp,
                              uint8_t              sync_flag,
                              uint16_t             divider_n1,
                              uint16_t             divider_n2);

int stfp_set_input_time_code_format(struct stfp_device*  stfp,
                                    uint8_t              format);


int stfp_set_input_time_code_modulation_type(struct stfp_device*  stfp,
                                             uint8_t              modulation);

void  stfp_set_propagation_delay_compensation(struct stfp_device*  stfp,
                                              int32_t              offset);

int stfp_request_data_timing_mode(struct stfp_device*  stfp,
                                  uint8_t*             timing_mode);

int stfp_request_data_time_format(struct stfp_device*  stfp,
                                  uint8_t*             time_format);

int stfp_request_data_current_year(struct stfp_device*  stfp,
                                   uint16_t*            year);

int stfp_request_data_periodic_output(struct stfp_device*  stfp,
                                      uint8_t*             synch,
                                      uint16_t*            divider_n1,
                                      uint16_t*            divider_n2);

int stfp_request_data_time_code_format(struct stfp_device*  stfp,
                                       uint8_t*             format);

int stfp_request_data_time_code_modulation(struct stfp_device*  stfp,
                                           uint8_t*             modulation);

int stfp_request_data_timing_offset(struct stfp_device*  stfp,
                                    int32_t*             offset);

int stfp_request_data_selected_clock_source(struct stfp_device*  stfp,
                                            uint8_t*             clock_source);

int stfp_request_data_firmware_part_number(struct stfp_device*  stfp,
                                           char*                firmware_part);

int stfp_request_data_assembly_part_number(struct stfp_device*  stfp,
                                           uint16_t*            assembly_part);

int stfp_request_data_fab_part_number(struct stfp_device*  stfp,
                                      uint16_t*            fab_part);

int stfp_request_data_model_identification(struct stfp_device*  stfp,
                                           char*                model);

int stfp_request_data_serial_number(struct stfp_device*  stfp,
                                    uint32_t*            serial_number);

void stfp_software_reset(struct stfp_device*  stfp);

int stfp_select_clock_source(struct stfp_device*  stfp,
                             uint8_t              clock_source);

int stfp_control_jam_sync(struct stfp_device*  stfp,
                          uint8_t              sync);

void stfp_force_jam_sync(struct stfp_device*  stfp);

void stfp_control_clock_disciplining(struct stfp_device*  stfp,
                                     uint8_t              enable);

void stfp_load_da_converter(struct stfp_device*  stfp,
                            uint16_t             value);

void stfp_set_disciplining_gain(struct stfp_device*  stfp,
                                int16_t              gain);

int stfp_register_interrupt_function(struct stfp_device*  stfp,
                                     void (*interrupt_function)(void*),
                                     void*  interrupt_function_argument);


#define DEVICE_MASK 0x0000ffff
#define VENDOR_MASK 0xffff0000

#define SELTIMINGMODE	1	
#	define MODE_TIMECODE	0
#	define MODE_FREERUN		1
#	define MODE_EXT1PPS		2
#	define MODE_GPS			6

#define SELTIMEFORMAT	2	
#	define TIME_DECIMAL	0
#	define TIME_BINARY	1

#define TIMEREQUEST		3
#define EVENTREQUEST	4

#define RDTIME			5
#define RDEVENT			6

#define RDTIMETV		7
#define RDEVENTTV		8

#define RDTIMEREQ		9
#define RDEVENTREQ		10

#define RDTIMETVREQ		11
#define RDEVENTTVREQ	12

#define WRSTROBE		13
#define WRSTROBETV		14

#define SELTCFORMAT		15
#	define TC_IRIGA		'A'
#	define TC_IRIGB		'B'
#	define TC_2137		'C'
#	define TC_NASA36	'N'
#	define TC_XR3		'X'

#define SELTCMOD		16
#	define MOD_AM		'M'
#	define MOD_DCLS		'D'

#define SETTIME			17
#define SETTIMETV		18
#define SETYEAR			19
#define SETLOCALOFFSET	20

#define SETPERIODIC		21
#	define PERIODIC_NOSYNC	0
#	define PERIODIC_SYNC	1

#define SETTIMINGOFFSET	22

#define SELFREQUENCYOUT	23
#	define FREQ_10MHZ		1
#	define FREQ_5MHZ		2
#	define FREQ_1MHZ		3

#define SELFCONN		24
#	define FCONN_EVENT_IN	0x00
#	define FCONN_1PPS_IN	0x01
#	define FCONN_DCLS_IN	0x02
#	define FCONN_STR2EV		0x03
#	define FCONN_1PPS_OUT	0x08
#	define FCONN_FREQ_OUT	0x09
#	define FCONN_PER_OUT	0x0A
#	define FCONN_STROBE_OUT	0x0B
#	define FCONN_DCLS_OUT	0x0C

#define CONTROLEVENT	25
#	define EVENT_RISING		1
#	define EVENT_FALLING	2
#	define CAPLOCK_ENABLE	3
#	define CAPLOCK_DISABLE	4
#	define EVCAP_ENABLE		5
#	define EVCAP_DISABLE	6
#	define EVCAP_PERIODIC	7
#	define EVCAP_EVENT		8

#define CONTROLSTROBE	26
#	define STROBE_SECUS		1
#	define STROBE_USONLY	2
#	define STROBE_ENABLE	3
#	define STROBE_DISABLE	4

#define CAPUNLOCK		27

#define SETINTSIGNAL	28
#define RDINTSIGNAL		29

#define RDINTSTAT		30
#define CLRINTSTAT		31

#define RDFIRMVERSION	32

#define CONTROLTIMEBASE	33
#	define CLOCK_INTERNAL	1
#	define CLOCK_EXTERNAL	2
#	define JAMSYNC_ENABLE	3
#	define JAMSYNC_DISABLE	4
#	define DISCIPLINE_ENABLE	5
#	define DISCIPLINE_DISABLE	6
#	define FORCE_JAMSYNC	7
#	define CLEAR_USEC		8

#define SETDAC			34
#define RDDAC			35

#define SETDISCGAIN		36

#define REQGPSPACKET	37

#define GETDATA	38
#	define 	GETDATA_MODE	0x10
#	define 	GETDATA_TFORMAT	0x11
#	define 	GETDATA_YEAR	0x13
#	define 	GETDATA_PER	0x14
#	define 	GETDATA_TCFMT	0x15
#	define 	GETDATA_TCMOD	0x16
#	define 	GETDATA_TOFF	0x17
#	define 	GETDATA_UTCINFO	0x18
#	define 	GETDATA_FWVER	0x1F
#	define 	GETDATA_CLKSRC	0x20
#	define 	GETDATA_JAMSC	0x21
#	define 	GETDATA_OSCCTL	0x23
#	define 	GETDATA_DAVAL	0x24

/*-------------------------------------------------------------*/
/* bc635S DPRAM Commands */

#define COM_SETTIMINGMODE	0x10
#define COM_SETTIMEFORMAT	0x11
#define COM_SETTIME			0x12
#define COM_SETYEAR			0x13
#define COM_SETPERIODIC		0x14
#define COM_SETTCFORMAT		0x15
#define COM_SETTCMOD		0x16
#define COM_SETTIMINGOFFSET	0x17
#define COM_GETDATA			0x19
#	define GETDATA_DAC		0
#	define GETDATA_VERSION	2
#define COM_SELCLOCK		0x20
#	define SELCLOCK_INT		'I'
#	define SELCLOCK_EXT		'E'
#define COM_JAMSYNCENABLE	0x21
#	define JAMSYNC_DIS		0
#	define JAMSYNC_EN		1
#define COM_FORCEJAMSYNC	0x22
#define	COM_DISCENABLE		0x23
#	define DISC_DIS			0
#	define DISC_EN			1
#define COM_SETDAC			0x24
#define COM_SETDISCGAIN		0x25
#define COM_SENDGPSPACKET	0x30
#define COM_REQGPSPACKET	0x31
#define COM_MANREQGPSPACKET	0x32
#define COM_GPSTIMEFORMAT	0x33

/*-------------------------------------------------------------*/
/* bc635S STATUS bits (from TIME0, EVENT0 registers) */

#define STAT_FLYWHEELING	0x01
#define STAT_TIME_OFFSET	0x02
#define STAT_FREQ_OFFSET	0x04

/*-------------------------------------------------------------*/
/* bc635S CONRTOL register bits */

#define CTRL_LOCKEN			0x01
#define CTRL_EVSOURCE		0x02
#define CTRL_EVSENSE		0x04
#define CTRL_EVENTEN		0x08
#define CTRL_STREN			0x10
#define CTRL_STRMODE		0x20
#define CTRL_F10			0x00
#define CTRL_F5				0x40
#define CTRL_F1				0x80
#define CTRL_FMASK			0xC0

/*-------------------------------------------------------------*/
/* bc635S ACK register bits */

#define ACK_COMMAND		0x01
#define ACK_PACKET		0x04
#define ACK_ACTION		0x80

/*-------------------------------------------------------------*/
/* bc635S INTSTAT and MASK register bits */
		
#define INT_EVENT		0x01
#define INT_PERIODIC	0x02
#define INT_STROBE		0x04
#define INT_1PPS		0x08
#define INT_PACKET		0x10
#define INT_BITMASK		0x1F


#define BC_SRESET               0x1A
#define COM_UTCINFO             0x18
#define COM_FIRMVER             0x1F
